@@ -3,7 +3,7 @@
  */
 
 /*
- * Copyright (C) 1986, 1988, 1989, 1991-2001, 2003, 2010-2013, 2017,
+ * Copyright (C) 1986, 1988, 1989, 1991-2001, 2003, 2010-2013, 2017, 2018,
  * the Free Software Foundation, Inc.
  *
  * This file is part of GAWK, the GNU implementation of the
@@ -101,6 +101,8 @@ err(bool isfatal, const char *s, const char *emsg, va_list argp)
 
 	if (isfatal) {
 #ifdef GAWKDEBUG
+		// GLIBC 2.27 doesn't necessarily flush on abort. Sigh.
+		fflush(NULL);
 		abort();
 #endif
 		gawk_exit(EXIT_FATAL);
